;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Tank Ardad. A tank videogame.
;;  Copyright (C) 2018 Adrian Bardisa Serrano, Ariel Palma Arriaga, David Ruiz Falco
;;  abs90@alu.ua.es drf7@alu.ua.es acpa2@alu.ua.es
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

;; Include all CPCtelera constant definitions, macros and variables
.include "cpctelera.h.s"
.include "entity.h.s"
.include "main.h.s"
.include "map.h.s"
.include "menu.h.s"
;;
;; Start of _DATA area 
;;  SDCC requires at least _DATA and _CODE areas to be declared, but you may use
;;  any one of them for any purpose. Usually, compiler puts _DATA area contents
;;  right after _CODE area contents.
;;
.area _DATA
.area _CODE





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;; Declare all function entry points as global symbols for the compiler.
;; (The linker will know what to do with them)
;; WARNING: Every global symbol declared will be linked, so DO NOT declare 
;; symbols for functions you do not use.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;
;; Start of _CODE area
;; 



;;
;; MAIN function. This is the entry point of the application.
;;    _main:: global symbol is required for correctly compiling and linking
;;
_main::
    
    ld l, #0
    call cpct_setSeed_lcg_u8_asm ;Poner el seed de random
    
    ld sp, #0x8000 ;Cambio el puntero de la pila para que no me destruya el backbuffer
    ;; Disable firmware to prevent it from interfering with string drawing
    call cpct_disableFirmware_asm ;; Deshabilita el firmware
    ld c,#0
    call cpct_setVideoMode_asm ;; Cambia el modo de video
    
    ld hl, #_g_palette ;;Carga el puntero a la paleta en HL
    ld de, #16         ;;Numero de colores de la paleta
    call cpct_setPalette_asm ;;Carga la paleta de colores

    ld l, #0x16
    ld h, #0x14 ; Color de fondo
    call cpct_setPALColour_asm

reiniciar:
    call ren_clearBackBuffer
    call load_menu

    ld ix, (m_next_level)
    ld b, 1(ix)
    ld c, 0(ix)
    call draw_map_front ;;Dibuja el mapa1
    ld ix, (m_next_level)
    ld b, 1(ix)
    ld c, 0(ix)
    call draw_map_back ;;Dibuja el mapa1
loop2:
    call load_hearths

    ld a, (difficulty)
    or a
    call z,initialize_enemies_map1
    dec a
    call z,initialize_enemies_map2
    dec a
    call z,initialize_enemies_map3
    dec a
    call z,initialize_enemies_map4

loop:
    call respawn_enemies

    ld ix,#hero_shot
    call ren_clear_backbuffer
    call ent_update_draw_shoot ;; Comprueba si la bala se ha disparado antes de dibujarla
    call ren_updateEntity

    ;;Vector de disparos de enemigos
    ld ix, #entity_vector_shots
    ld hl, #ren_clear_backbuffer 
    call ent_doForAll
    ld hl, #ent_update_draw_shoot
    call ent_doForAll
    ld hl, #ren_updateEntity
    call ent_doForAll


    ;;Vector de enemigos
    ld ix, #entity_vector
    ld hl, #ren_clear_backbuffer 
    call ent_doForAll
    ld hl, #ent_update
    ld a, (unavariable)
    or a
    call z,ent_doForAll
    ld hl, #ren_updateEntity
    ld a, (unavariable)
    or a
    call z,ent_doForAll
    ld hl, #ent_draw
    call ent_doForAll
    ld hl, #ex_sprite_spriteB
    call ent_doForAll

    call load_explosion

    ld ix,#hero_data
    call ren_clear_backbuffer
    ld a, (unavariable)
    or a
    call z,ent_update
    call ren_updateEntity
    call ent_draw
    
    ld ix, #hero_shot
    ld a, e_eye(ix)
    or a
    jr z, space_no_pulsada ;; Si e_eye(ix)=0 significa que ya hay una bala del heroe
    ld hl, #Key_Space      ;; disparandose y no pueden haber dos disparos del mismo tanque a la vez
    call cpct_isKeyPressed_asm
    jr z, space_no_pulsada
    call ent_shot

space_no_pulsada:
    call cpct_waitVSYNC_asm
    call ren_switchBuffers
    ld a, (unavariable)
    xor #1
    ld (unavariable), a

    ld a, (m_num_ent)
    or a
    jr z, next_level

    ld ix, #hero_data
    ld a, e_hp(ix)
    or a
    jr z, gameOver
    jp loop

next_level:
    call load_next_level

    jp loop2
    
gameOver:
    call load_game_over
    jp reiniciar

unavariable: .db #1