;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Tank Ardad. A tank videogame.
;;  Copyright (C) 2018 Adrian Bardisa Serrano, Ariel Palma Arriaga, David Ruiz Falco
;;  abs90@alu.ua.es drf7@alu.ua.es acpa2@alu.ua.es
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "render.h.s"
.include "main.h.s"

m_front_buffer:: .db 0xC0  ;C000 principio de la memoria de video
m_back_buffer:: .db 0x80 ;8000 final de la memoria de video

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;LIMPIA EL BACK BUFFER
;;DESTROYS: BC, DE, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ren_newScene:
	call ren_switchBuffers
	call ren_clearBackBuffer

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;INICIALIZAR EL BUFFER A 0
;;DESTROYS: BC, DE, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ren_clearBackBuffer:
	ld hl,#0x8000
	ld de,#0x8001
	ld (hl),#0x00
	ld bc,#4000
	ldir
	ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;INTERCAMBIA LOS BUFFERS
;;DESTROYS: A, B, L
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ren_switchBuffers:
	ld a, (m_back_buffer)
	ld b,a
	ld a, (m_front_buffer)
	ld (m_back_buffer), a
	ld a, b
	ld (m_front_buffer), a

	srl b ; desplazamiento de un registro a la derecha
	srl b

	ld l, b
	call cpct_setVideoMemoryPage_asm ;Pone el puntero de la memoria de video apuntando al otro buffer
ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;entity Draw 
;;INPUT: IX -> puntero a entidad
;;DESTROYS: BC, DE, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ren_drawEntity:
	ld a,(m_back_buffer) ; DE = Back buffer
	ld d, a
	ld e, #0
    ;;;Comienzo la memoria de video
    ld c,de_x(ix)                ;;; C=entity x
    ld b,de_y(ix)                ;;; C=entity y
    call cpct_getScreenPtr_asm

    ex de,hl           ;; intercambia el valor de hl a de , en hl esta la posicion colocada
    ld h,de_sprite_h(ix)
    ld l,de_sprite_l(ix)
    ld bc,#0x0804      ;;c->ancho, b->alto en bytes
    call cpct_drawSprite_asm
ret


ren_drawEntity_front:
    ld a,(m_front_buffer) ; DE = Back buffer
    ld d, a
    ld e, #0
    ;;;Comienzo la memoria de video
    ld c,de_x(ix)                ;;; C=entity x
    ld b,de_y(ix)                ;;; C=entity y
    call cpct_getScreenPtr_asm

    ex de,hl           ;; intercambia el valor de hl a de , en hl esta la posicion colocada
    ld h,de_sprite_h(ix)
    ld l,de_sprite_l(ix)
    ld bc,#0x0804      ;;c->ancho, b->alto en bytes
    call cpct_drawSprite_asm
ret

ren_draw_menu::
	ld a,(m_front_buffer) ; DE = Back buffer
	ld d, a
	ld e, #0
    ;;;Comienzo la memoria de video
    ld c,de_x(ix)                ;;; C=entity x
    ld b,de_y(ix)                ;;; C=entity y
    call cpct_getScreenPtr_asm

    ex de,hl           ;; intercambia el valor de hl a de , en hl esta la posicion colocada
    ld h,de_sprite_h(ix)
    ld l,de_sprite_l(ix)
    ld bc,#0x0804      ;;c->ancho, b->alto en bytes
    call cpct_drawSprite_asm
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;entity clear
;;INPUT: IX -> puntero a entidad
;;DESTROYS: HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ren_clearEntity:
    ld h, de_sprite_h(ix)
    ld l, de_sprite_l(ix)
    push hl
    ld hl, #_sprite_BlackTile
    ld de_sprite_h(ix), h
    ld de_sprite_l(ix), l
    call ren_drawEntity
    pop hl
    ld de_sprite_h(ix), h
    ld de_sprite_l(ix), l 
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;entity clear
;;INPUT: IX -> puntero a entidad
;;DESTROYS: HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ren_clear_backbuffer:
    ld h, de_sprite_h(ix)
    ld l, de_sprite_l(ix)
    push hl
    ld hl, #_sprite_BlackTile
    ld de_sprite_h(ix), h
    ld de_sprite_l(ix), l
    call ren_draw_backbuffer
    pop hl
    ld de_sprite_h(ix), h
    ld de_sprite_l(ix), l 
    ret

ren_clear_frontbuffer:
    ld h, de_sprite_h(ix)
    ld l, de_sprite_l(ix)
    push hl
    ld hl, #_sprite_BlackTile
    ld de_sprite_h(ix), h
    ld de_sprite_l(ix), l
    call ren_draw_frontbuffer
    pop hl
    ld de_sprite_h(ix), h
    ld de_sprite_l(ix), l 
    ret
ren_draw_frontbuffer:

    ld a,(m_front_buffer) ; DE = Back buffer
    ld d, a
    ld e, #0
    ;;;Comienzo la memoria de video
    ld c,de_x(ix)                ;;; C=entity x
    ld b,de_y(ix)                ;;; C=entity y
    call cpct_getScreenPtr_asm

    ex de,hl           ;; intercambia el valor de hl a de , en hl esta la posicion colocada
    ld h,de_sprite_h(ix)
    ld l,de_sprite_l(ix)
    ld bc,#0x0804      ;;c->ancho, b->alto en bytes
    call cpct_drawSprite_asm
ret

ren_draw_blacktile_explosion:
    ld a,(m_front_buffer) ; DE = Back buffer
    ld d, a
    ld e, #0
    ;;;Comienzo la memoria de video
    ld c,de_x(ix)                ;;; C=entity x
    ld b,de_y(ix)                ;;; C=entity y
    call cpct_getScreenPtr_asm

    ex de,hl           ;; intercambia el valor de hl a de , en hl esta la posicion colocada
    ld h,de_sprite_h(ix)
    ld l,de_sprite_l(ix)
    ld bc,#0x0804      ;;c->ancho, b->alto en bytes
    call cpct_drawSprite_asm
    
    ld a,(m_back_buffer) ; DE = Back buffer
    ld d, a
    ld e, #0
    ;;;Comienzo la memoria de video
    ld c,e_xaa(ix)                ;;; C=entity x
    ld b,e_yaa(ix)                ;;; C=entity y
    call cpct_getScreenPtr_asm

    ex de,hl           ;; intercambia el valor de hl a de , en hl esta la posicion colocada
    ld h,de_sprite_h(ix)
    ld l,de_sprite_l(ix)
    ld bc,#0x0804      ;;c->ancho, b->alto en bytes
    call cpct_drawSprite_asm
ret

ren_draw_backbuffer:

    ld a,(m_back_buffer) ; DE = Back buffer
    ld d, a
    ld e, #0
    ;;;Comienzo la memoria de video
    ld c,e_xaa(ix)                ;;; C=entity x
    ld b,e_yaa(ix)                ;;; C=entity y
    call cpct_getScreenPtr_asm

    ex de,hl           ;; intercambia el valor de hl a de , en hl esta la posicion colocada
    ld h,de_sprite_h(ix)
    ld l,de_sprite_l(ix)
    ld bc,#0x0804      ;;c->ancho, b->alto en bytes
    call cpct_drawSprite_asm
ret

ren_clear_menu::
    ld h, de_sprite_h(ix)
    ld l, de_sprite_l(ix)
    push hl
    ld hl, #_sprite_BlackTile
    ld de_sprite_h(ix), h
    ld de_sprite_l(ix), l
    call ren_draw_menu
    pop hl
    ld de_sprite_h(ix), h
    ld de_sprite_l(ix), l 
    ret

;ren_updateEntity:
;    ;update X
;    ld a, de_x1(ix)
;    ld de_x2(ix), a
;    ld a, de_x0(ix)
;    ld de_x1(ix), a
;
;    ;;update Y
;    ld a, de_y1(ix)
;    ld de_y2(ix), a
;    ld a, de_y0(ix)
;    ld de_y1(ix), a;
ren_updateEntity:
    ;update X
    ld a, e_xa(ix)
    ld e_xaa(ix), a
    ld a, de_x(ix)
    ld e_xa(ix), a

    ;;update Y
    ld a, e_ya(ix)
    ld e_yaa(ix), a
    ld a, de_y(ix)
    ld e_ya(ix), a
    ret